@@ -7,10 +7,10 @@ use strict;
 
 package XML::Compile::Iterator;
 use vars '$VERSION';
-$VERSION = '1.27';
+$VERSION = '1.28';
 
 
-use XML::Compile::Util  qw/pack_type type_of_node/;
+use XML::Compile::Util  qw/pack_type type_of_node SCHEMA2001i/;
 use Log::Report 'xml-compile', syntax => 'SHORT';
 
 
@@ -111,6 +111,13 @@ sub nodeLocal()
 }
 
 
+sub nodeNil()
+{   my $node = shift->node or return 0;
+    my $nil  = $node->getAttributeNS(SCHEMA2001i, 'nil') || '';
+    $nil eq 'true' || $nil eq '1';
+}
+
+
 sub textContent()
 {   my $node = shift->node or return undef;
     $node->textContent;
