@@ -8,7 +8,7 @@ no warnings 'recursion';  # trees can be quite deep
 
 package XML::Compile::Translate;
 use vars '$VERSION';
-$VERSION = '1.27';
+$VERSION = '1.28';
 
 
 # Errors are either in _class 'usage': called with request
@@ -636,11 +636,6 @@ sub element($)
     my $is_simple = defined $st;
     my $nillable  = $self->isTrue($node->getAttribute('nillable') || 'false');
 
-    if(!$nillable) {}
-    elsif($is_simple)
-         { $st    = $self->makeNillableSimple($where, $nodetype, $st,  $tag) }
-    else { $elems = $self->makeNillableComplex($where,$nodetype,$elems,$tag) }
-
     my $elem_handler
       = $comps->{mixed}          ? 'makeMixedElement'
       : ! $is_simple             ? 'makeComplexElement' # other complexType
@@ -648,7 +643,7 @@ sub element($)
       :                            'makeSimpleElement';
 
     my $r = $self->$elem_handler
-       ($where, $tag, ($st||$elems), $attrs, $attrs_any, $comptype);
+       ( $where, $tag, ($st||$elems), $attrs, $attrs_any, $comptype, $nillable);
 
     # Add defaults and stuff
     my $default  = $node->getAttributeNode('default');
@@ -982,6 +977,12 @@ sub prefixed($)
     length $pn->{prefix} ? "$pn->{prefix}:$local" : $local;
 }
 
+sub namespaceForPrefix($)
+{   my ($self, $prefix) = @_;   # only rarely used
+    my $match = first {$_->{prefix} eq $prefix} values %{$self->{prefixes}};
+    $match ? $match->{uri} : undef;
+}
+
 sub attributeOne($)
 {   my ($self, $tree) = @_;
 
@@ -1405,6 +1406,7 @@ sub complexContent($$)
     # attributes: id, mixed = boolean
     # content: annotation?, (restriction | extension)
 
+my $path = $tree->path;
     my $node = $tree->node;
     if(my $m = $node->getAttribute('mixed'))
     {   $mixed = $self->isTrue($m)
